@@ -8,7 +8,7 @@ __copyright__ = "(C) 2010 Science and Technology Facilities Council"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
 __license__ = "BSD - see LICENSE file in top-level directory"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = "$Id: response.py 7087 2010-06-25 11:23:09Z pjkersha $"
+__revision__ = "$Id: response.py 8010 2012-01-30 16:24:06Z rwilkinson $"
 import logging
 log = logging.getLogger(__name__)
 
@@ -42,4 +42,20 @@ class Response(XacmlContextBase):
         @rtype: ndg.xacml.utils.TypedList
         """
         return self.__results
-    
+
+    def __getstate__(self):
+        '''Enable pickling
+        
+        @return: object's attribute dictionary
+        @rtype: dict
+        '''
+        _dict = super(Response, self).__getstate__()
+        for attrName in Response.__slots__:
+            # Ugly hack to allow for derived classes setting private member
+            # variables
+            if attrName.startswith('__'):
+                attrName = "_Response" + attrName
+                
+            _dict[attrName] = getattr(self, attrName)
+            
+        return _dict
