@@ -8,7 +8,7 @@ __copyright__ = "(C) 2010 Science and Technology Facilities Council"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
 __license__ = "BSD - see LICENSE file in top-level directory"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = "$Id: rule_combining_alg.py 7682 2010-11-01 16:30:22Z pjkersha $"
+__revision__ = "$Id: rule_combining_alg.py 7955 2011-12-21 18:29:45Z rwilkinson $"
 import logging
 log = logging.getLogger(__name__)
 
@@ -167,6 +167,47 @@ class PermitOverridesRuleCombiningAlg(RuleCombiningAlgInterface):
         return Decision.NOT_APPLICABLE
 
     
+class FirstApplicableRuleCombiningAlg(RuleCombiningAlgInterface):
+    """Implementation of first applicable XACML rule combining algorithm"""
+    
+    def evaluate(self, rules, context):
+        """Combine the input rule results to make an access control decision.
+        Implementation taken direct from XACML 2.0 spec. pseudo code - Section
+        C.5
+        
+        @param rules: rules from the policy.  Decisions from these will be put
+        together into a single decision by this algorithm
+        @type rules: TypedList(<ndg.xacml.core.rule.Rule>)
+        @param context: request context to apply to the rules
+        @type context: ndg.xacml.core.request.Request
+        @return: resulting overall access control decision
+        @rtype: ndg.xacml.core.context.result.Decision
+        """
+        for rule in rules:
+            decision = rule.evaluate(context)
+            if decision == Decision.DENY:
+                log.debug("Rule %r denies, returning overall deny decision",
+                          rule.id)
+                return Decision.DENY
+
+            if decision == Decision.PERMIT:
+                log.debug("Rule %r permits, returning overall permit decision",
+                          rule.id)
+                return Decision.PERMIT
+
+            if decision == Decision.NOT_APPLICABLE:
+                continue
+
+            if decision == Decision.INDETERMINATE:
+                log.debug("Rule %r is indeterminate, returning overall "
+                          "indeterminate decision", rule.id)
+                return Decision.INDETERMINATE
+        
+        log.debug('No rules were applicable to the request, returning '
+                  'overall not applicable decision')
+        return Decision.NOT_APPLICABLE
+
+
 class RuleCombiningAlgClassFactory(object):
     """Class Factory mapping Rule Combining Algorithm identifiers to their
     class implementations"""
@@ -179,7 +220,9 @@ class RuleCombiningAlgClassFactory(object):
     'urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:deny-overrides':
         DenyOverridesRuleCombiningAlg,      
     'urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:permit-overrides':
-        PermitOverridesRuleCombiningAlg
+        PermitOverridesRuleCombiningAlg,
+    'urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:first-applicable':
+        FirstApplicableRuleCombiningAlg
     })
     __slots__ = ('__map',)
     
