@@ -14,7 +14,7 @@ __copyright__ = "(C) 2010 Science and Technology Facilities Council"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
 __license__ = "BSD - see LICENSE file in top-level directory"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = "$Id: target.py 7260 2010-08-02 12:28:59Z pjkersha $"
+__revision__ = "$Id: target.py 7955 2011-12-21 18:29:45Z rwilkinson $"
 import logging
 log = logging.getLogger(__name__)
 
@@ -194,6 +194,6 @@ class Target(XacmlCoreBase):
         for i, childMatch in enumerate(targetChild.matches):
             matchStatusValues[i] = childMatch.evaluate(request)
             
-        # Any match => overall match      
-        return any(matchStatusValues)
+        # All match => overall match      
+        return all(matchStatusValues)
 
