@@ -8,20 +8,21 @@ __copyright__ = "(C) 2010 Science and Technology Facilities Council"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
 __license__ = "BSD - see LICENSE file in top-level directory"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = "$Id: applyreader.py 7661 2010-10-27 15:05:18Z pjkersha $"
+__revision__ = "$Id: applyreader.py 8041 2012-03-16 11:51:01Z rwilkinson $"
 import logging
 log = logging.getLogger(__name__)
 
 from ndg.xacml.core.apply import Apply
 from ndg.xacml.core.attributevalue import AttributeValue
+from ndg.xacml.core.condition import Condition
 from ndg.xacml.core.variablereference import VariableReference
 from ndg.xacml.core.attributeselector import AttributeSelector
 from ndg.xacml.core.attributedesignator import (SubjectAttributeDesignator,
-                                                EnvironmentAttributeDesignator,
+                                                ResourceAttributeDesignator,
                                                 ActionAttributeDesignator,
                                                 EnvironmentAttributeDesignator)
 from ndg.xacml.parsers import XMLParseError
-from ndg.xacml.parsers.etree import QName
+from ndg.xacml.parsers.etree import QName, getElementChildren
 from ndg.xacml.parsers.etree.reader import ETreeAbstractReader
 from ndg.xacml.parsers.etree.factory import ReaderFactory
 
@@ -47,7 +48,7 @@ class ApplyReader(ETreeAbstractReader):
     FUNCTION_ELEMENT_LOCAL_NAME = 'Function'
     VARIABLE_REFERENCE_ELEMENT_LOCAL_NAME = 'VariableReference'
     
-    def __call__(self, obj):
+    def __call__(self, obj, common):
         """Parse Apply type object
 
         @param obj: input object to parse
@@ -79,58 +80,72 @@ class ApplyReader(ETreeAbstractReader):
         
         # Allow for any of the defined Expression sub-types in the child 
         # elements
-        for subElem in elem:
+        for subElem in getElementChildren(elem):
             localName = QName.getLocalPart(subElem.tag)
             if localName == xacmlType.ELEMENT_LOCAL_NAME:
-                applyObj.expressions.append(ApplyReader.parse(subElem))
+                applyObj.expressions.append(ApplyReader.parse(subElem, common))
              
             elif localName == AttributeValue.ELEMENT_LOCAL_NAME:
                 AttributeValueReader = ReaderFactory.getReader(AttributeValue) 
-                applyObj.expressions.append(AttributeValueReader.parse(subElem))
+                applyObj.expressions.append(AttributeValueReader.parse(subElem,
+                                                                       common))
                  
             elif localName == SubjectAttributeDesignator.ELEMENT_LOCAL_NAME:
                 SubjectAttributeDesignatorReader = ReaderFactory.getReader(
                                                 SubjectAttributeDesignator)
                 applyObj.expressions.append(
-                                SubjectAttributeDesignatorReader.parse(subElem))
+                                SubjectAttributeDesignatorReader.parse(subElem,
+                                                                       common))
+                
+            elif localName == ResourceAttributeDesignator.ELEMENT_LOCAL_NAME:
+                ResourceAttributeDesignatorReader = ReaderFactory.getReader(
+                                                ResourceAttributeDesignator)
+                applyObj.expressions.append(
+                                ResourceAttributeDesignatorReader.parse(subElem,
+                                                                        common))
                 
             elif localName == EnvironmentAttributeDesignator.ELEMENT_LOCAL_NAME:
                 EnvironmentAttributeDesignatorReader = ReaderFactory.getReader(
                                                 EnvironmentAttributeDesignator)
                 applyObj.expressions.append(
-                            EnvironmentAttributeDesignatorReader.parse(subElem))
+                            EnvironmentAttributeDesignatorReader.parse(subElem,
+                                                                       common))
                 
             elif localName == ActionAttributeDesignator.ELEMENT_LOCAL_NAME:
                 ActionAttributeDesignatorReader = ReaderFactory.getReader(
                                                 ActionAttributeDesignator)
                 applyObj.expressions.append(
-                                ActionAttributeDesignatorReader.parse(subElem))
+                                ActionAttributeDesignatorReader.parse(subElem,
+                                                                      common))
                 
             elif localName == EnvironmentAttributeDesignator.ELEMENT_LOCAL_NAME:
                 EnvironmentAttributeDesignatorReader = ReaderFactory.getReader(
                                                 EnvironmentAttributeDesignator)
                 applyObj.expressions.append(
-                            EnvironmentAttributeDesignatorReader.parse(subElem))
+                            EnvironmentAttributeDesignatorReader.parse(subElem,
+                                                                       common))
                 
             elif localName == AttributeSelector.ELEMENT_LOCAL_NAME:
                 AttributeSelectorReader = ReaderFactory.getReader(
                                                             AttributeSelector)
                 applyObj.expressions.append(
-                                        AttributeSelectorReader.parse(subElem))
+                                        AttributeSelectorReader.parse(subElem,
+                                                                      common))
             
             elif localName == Condition.ELEMENT_LOCAL_NAME:
                 ConditionReader = ReaderFactory.getReader(Condition)
-                applyObj.expressions.append(ConditionReader.parse(subElem))
+                applyObj.expressions.append(ConditionReader.parse(subElem,
+                                                                  common))
                 
             elif localName == self.__class__.FUNCTION_ELEMENT_LOCAL_NAME:
                 raise NotImplementedError('%r Apply sub-element not '
                                           'implemented', localName)
             
-            elif (localName == VariableReference.ELEMENT_LOCAL_NAME):                                
+            elif (localName == VariableReference.ELEMENT_LOCAL_NAME):
                 raise NotImplementedError('%r Apply sub-element not '
                                           'implemented', localName)
             else:
                 raise XMLParseError('%r Apply sub-element not recognised', 
                                     localName)
    
-        return applyObj
\ No newline at end of file
+        return applyObj
